package edu.ntnu.stud.chaosgame.view;

import edu.ntnu.stud.chaosgame.controller.game.ChaosCanvas;
import edu.ntnu.stud.chaosgame.controller.game.ChaosGame;
import edu.ntnu.stud.chaosgame.controller.game.ChaosGameDescription;
import edu.ntnu.stud.chaosgame.model.generators.ChaosGameDescriptionFactory;

import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;
import javafx.util.Duration;

import java.io.IOException;
import java.util.concurrent.atomic.AtomicReference;

public class ChaosGameGUIView {
    private int currentLine = 0;

    private PixelWriter pixelWriter;

    private ImageView imageView;
    private int width;
    private int height;

    private int canvasWidth;
    private int canvasHeight;
    private ChaosGame game;

    private final Timeline timeline;



    public ChaosGameGUIView(Stage primaryStage) throws IOException {

        this.timeline = new Timeline(new KeyFrame(Duration.seconds(2), event -> this.drawChaosGame()));


        imageView = new ImageView();
        width = 1000;
        height = 1000;

        canvasWidth = 1000;
        canvasHeight = 1000;

        WritableImage writableImage = new WritableImage(width, height);
        pixelWriter = writableImage.getPixelWriter();

        imageView.setImage(writableImage);

        imageView.setFitHeight(width);
        imageView.setFitWidth(height);

        VBox sideMenu = new VBox();


        //TEMPORARY CODE to test Chaos Games in GUI

        ChaosGameDescriptionFactory factory = new ChaosGameDescriptionFactory();
        AtomicReference<ChaosGameDescription> description = new AtomicReference<>(factory.getDescriptions().get(0));
        ChaosCanvas canvas = new ChaosCanvas(canvasWidth, canvasHeight, description.get().getMinCoords(), description.get().getMaxCoords());
        game = new ChaosGame(description.get(), canvas);

        Button startButton = new Button("Start");
        startButton.setOnAction(event -> this.drawChaosGame());
        Button stopButton = new Button("Stop");
        stopButton.setOnAction(event -> timeline.stop());


        Button newButton = new Button("New");

        newButton.setOnAction(event ->{
            WritableImage newWritableImage = new WritableImage(width, height);
            setPixelWriter(newWritableImage.getPixelWriter());
            setImageViewFromImage(newWritableImage);
            canvas.clearCanvas();
        });

        Button clearButton = new Button("Clear");

        clearButton.setOnAction(event -> {
            getImageView().setImage(null);
            setCurrentLine(0);
        });
        
        
        // Radio buttons for choosing fractal type
        ToggleGroup group = new ToggleGroup();
        RadioButton sierpinskiRadioButton = new RadioButton("Sierpinski");
        sierpinskiRadioButton.setToggleGroup(group);
        sierpinskiRadioButton.setSelected(true);
        RadioButton barnsleyRadioButton = new RadioButton("Barnsley");
        barnsleyRadioButton.setToggleGroup(group);
        RadioButton juliaRadioButton = new RadioButton("Julia");
        juliaRadioButton.setToggleGroup(group);

        AtomicReference<ChaosCanvas> canvasRef = new AtomicReference<>(canvas);
        sierpinskiRadioButton.setOnAction(event -> {
            timeline.stop();
            canvasRef.get().clearCanvas();
            WritableImage newWritableImage = new WritableImage(width, height);
            setPixelWriter(newWritableImage.getPixelWriter());
            setImageViewFromImage(newWritableImage);
            canvas.clearCanvas();

            description.set(factory.getDescriptions().get(0)); // Assuming the Sierpinski description is at index 0
            canvasRef.set(new ChaosCanvas(canvasWidth, canvasHeight, description.get().getMinCoords(), description.get().getMaxCoords()));
            game = new ChaosGame(description.get(), canvasRef.get());
        });

        barnsleyRadioButton.setOnAction(event -> {
            timeline.stop();
            canvasRef.get().clearCanvas();
            WritableImage newWritableImage = new WritableImage(width, height);
            setPixelWriter(newWritableImage.getPixelWriter());
            setImageViewFromImage(newWritableImage);
            canvas.clearCanvas();
            description.set(factory.getDescriptions().get(1)); // Assuming the Sierpinski description is at index 0
            canvasRef.set(new ChaosCanvas(canvasWidth, canvasHeight, description.get().getMinCoords(), description.get().getMaxCoords()));
            game = new ChaosGame(description.get(), canvasRef.get());
        });

        juliaRadioButton.setOnAction(event -> {
            timeline.stop();
            canvasRef.get().clearCanvas();
            WritableImage newWritableImage = new WritableImage(width, height);
            setPixelWriter(newWritableImage.getPixelWriter());
            setImageViewFromImage(newWritableImage);
            canvas.clearCanvas();
            description.set(factory.getDescriptions().get(2)); // Assuming the Sierpinski description is at index 0
            canvasRef.set(new ChaosCanvas(canvasWidth, canvasHeight, description.get().getMinCoords(), description.get().getMaxCoords()));
            game = new ChaosGame(description.get(), canvasRef.get());
        });

        // Load fractal file button
        Button loadFractalFromFileButton = new Button("Load Fractal");
        // Write fractal to file button
        Button writeFractalToFileButton = new Button("Write to File");

        // Parameters
        VBox parameterBox = new VBox();
        // Step Count GUI
        VBox stepCountBox = new VBox();
        Label stepCountLabel = new Label("Step Count");
        TextArea stepCountTextArea = new TextArea();
        stepCountTextArea.setPrefHeight(5);
        stepCountTextArea.setPrefWidth(50);
        Button changeStepCountButton = new Button("Change Step Count");
        stepCountBox.getChildren().addAll(stepCountLabel,stepCountTextArea, changeStepCountButton);
        // Minimum Coordinates GUI
        VBox minCoordinatesBox = new VBox();
        Label minCoordinatesLabel = new Label("Min. Coordinates");
        TextArea minimumCoordinatesTextArea = new TextArea();
        minimumCoordinatesTextArea.setPrefHeight(5);
        minimumCoordinatesTextArea.setPrefWidth(50);
        Button changeMinimumCoordinatesButton = new Button("Change Min. Coordinates");
        minCoordinatesBox.getChildren().addAll(minCoordinatesLabel,minimumCoordinatesTextArea,changeMinimumCoordinatesButton);
        // Maximum Coordinates GUI
        VBox maxCoordinatesBox = new VBox();
        Label maxCoordinatesLabel = new Label("Max Coordinates");
        TextArea maximumCoordinatesTextArea = new TextArea();
        maximumCoordinatesTextArea.setPrefHeight(5);
        maximumCoordinatesTextArea.setPrefWidth(50);
        Button changeMaximumCoordinatesButton = new Button("Change Max Coordinates");
        maxCoordinatesBox.getChildren().addAll(maxCoordinatesLabel,maximumCoordinatesTextArea,changeMaximumCoordinatesButton);
        // Fill parameter box
        parameterBox.getChildren().addAll(stepCountBox, minCoordinatesBox, maxCoordinatesBox);
        parameterBox.setPadding(new Insets(10));

        // Quit button
        Button quitButton = new Button("Quit");
        quitButton.setOnAction(event -> Platform.exit());

        // Add basic control buttons
        sideMenu.getChildren().addAll(startButton,stopButton,newButton,clearButton);
        // Add fractal radio buttons
        sideMenu.getChildren().addAll(sierpinskiRadioButton, barnsleyRadioButton, juliaRadioButton);
        // Add parameter VBox
        sideMenu.getChildren().add(parameterBox);
        // Add file buttons
        sideMenu.getChildren().addAll(loadFractalFromFileButton,writeFractalToFileButton);
        // Add quit button
        sideMenu.getChildren().add(quitButton);
        // Add padding
        sideMenu.setPadding(new Insets(10));



        BorderPane borderPane = new BorderPane();
        borderPane.setCenter(imageView);
        borderPane.setRight(sideMenu);

        Scene scene = new Scene(borderPane,1700,1000);

        primaryStage.setTitle("Fractal Chaos Game");
        primaryStage.setScene(scene);
        primaryStage.show();




    }

    public void drawChaosGame(){
        ChaosCanvas canvas = game.getCanvas();


        game.runSteps(1000000000);

        // Test implementation for drawing fractals
        int[][] betaArray = canvas.getCanvasArray();
        for (int i = 0; i < canvas.getWidth(); i++) {
            for (int j = 0; j < canvas.getHeight(); j++) {
                if (betaArray[i][j] == 1) {
                    pixelWriter.setColor(j,i,Color.BLACK);
                }
            }

        }



    }

    public int getWidth(){
        return this.width;
    }
    public int getHeight(){
        return this.height;
    }
    public ImageView getImageView(){
        return this.imageView;
    }

    public void setCurrentLine(int currentLine) {
        this.currentLine = currentLine;
    }

    public void setPixelWriter(PixelWriter pixelWriter) {
        this.pixelWriter = pixelWriter;
    }

    public void setImageViewFromImage(Image inputView) {
        this.imageView.setImage(inputView);
    }

}
